@@ -1,18 +1,20 @@
 #ifndef LPSQN_P_H
 #define LPSQN_P_H
 #include <cstddef>
+#include <algorithm>
+#include "constant.h"
 
 namespace lp {
 
-inline void copy(double * __restrict__ x, double const * __restrict__ y,
+inline void copy(double * PSQN_RESTRICT x, double const * PSQN_RESTRICT y,
                  size_t const dim) noexcept {
   for(size_t i = 0; i < dim; ++i, ++x, ++y)
     *x = *y;
 }
 
 inline void vec_diff
-(double const * __restrict__ x, double const * __restrict__ y,
- double * __restrict__ res, size_t const n) noexcept {
+(double const * PSQN_RESTRICT x, double const * PSQN_RESTRICT y,
+ double * PSQN_RESTRICT res, size_t const n) noexcept {
   for(size_t i = 0; i < n; ++i, ++x, ++y, ++res)
     *res = *x - *y;
 }
@@ -25,7 +27,7 @@ inline double vec_dot(double const *x, size_t const n) noexcept {
 }
 
 inline double vec_dot
-(double const * __restrict__ x, double const * __restrict__ y,
+(double const * PSQN_RESTRICT x, double const * PSQN_RESTRICT y,
  size_t const n) noexcept {
   double out(0.);
   for(size_t i = 0; i < n; ++i, ++x, ++y)
@@ -38,8 +40,8 @@ inline double vec_dot
  the upper triangular and x is a n-dimensional vector.
  */
 inline void mat_vec_dot
-(double const * __restrict__  X, double const * const __restrict__ x,
- double * const __restrict__ res, size_t const n) noexcept {
+(double const * PSQN_RESTRICT  X, double const * const PSQN_RESTRICT x,
+ double * const PSQN_RESTRICT res, size_t const n) noexcept {
   double const * xj = x;
   double * rj = res;
   for(size_t j = 0; j < n; ++j, ++xj, ++rj){
@@ -54,53 +56,55 @@ inline void mat_vec_dot
   }
 }
 
-/** util class which jumps from one memory location to another. */
-template<class T>
-class sep_mem {
-  T *       cur,
-    * const d1_end,
-    * const d2;
-public:
-  sep_mem(T * d1, T * d2, size_t const n1) noexcept:
-  cur(n1 > 0 ? d1 : d2), d1_end(d1 + n1), d2(d2) { }
-
-  inline sep_mem& operator++() noexcept {
-    if(++cur != d1_end)
-      return *this;
-
-    cur = d2;
-    return *this;
-  }
-
-  inline operator T*() const noexcept {
-    return cur;
-  }
-};
-
 /**
  computes b <- b + Xx where b and x are seperated into an nb1 and bn2
  dimensional vector.
  */
 inline void mat_vec_dot
-(double const *__restrict__ X, double const * __restrict__ x1,
- double const * __restrict__ x2, double * const __restrict__ r1,
- double * const __restrict__ r2,
+(double const * PSQN_RESTRICT X, double const * PSQN_RESTRICT x1,
+ double const * PSQN_RESTRICT x2, double * const PSQN_RESTRICT r1,
+ double * const PSQN_RESTRICT r2,
  size_t const n1, size_t const n2) noexcept {
-  sep_mem<double const> xj(x1, x2, n1);
-  sep_mem<double>       rj(r1, r2, n1);
-  auto const xi_start = xj;
-  auto const ri_start = rj;
-
   size_t const n = n1 + n2;
-  for(size_t j = 0; j < n; ++j, ++xj, ++rj){
-    sep_mem<double const> xi = xi_start;
-    sep_mem<double>       ri = ri_start;
+  auto loop_body =
+    [&](double const xj, double &rj, size_t const j) -> void {
+      size_t i = 0L;
+      {
+        double       * ri = r1;
+        double const * xi = x1;
+        size_t const iend = std::min(j, n1);
+        for(; i < iend; ++i, ++X, ++ri, ++xi){
+          *ri += *X *  xj;
+           rj += *X * *xi;
+        }
+        if(i < n1)
+          rj += *X++ * *xi;
+      }
 
-    for(size_t i = 0L; i < j; ++i, ++X, ++ri, ++xi){
-      *ri += *X * *xj;
-      *rj += *X * *xi;
-    }
-    *rj += *X++ * *xi;
+      if(i == n1){ // still work to do
+        double       * ri = r2;
+        double const * xi = x2;
+        for(; i < j; ++i, ++X, ++ri, ++xi){
+          *ri += *X *  xj;
+           rj += *X * *xi;
+        }
+        rj += *X++ * *xi;
+
+      }
+  };
+
+  {
+    double const *xj = x1;
+    double       *rj = r1;
+    for(size_t j = 0; j < n1; ++j, ++xj, ++rj)
+      loop_body(*xj, *rj, j);
+  }
+
+  {
+    double const *xj = x2;
+    double       *rj = r2;
+    for(size_t j = n1; j < n; ++j, ++xj, ++rj)
+      loop_body(*xj, *rj, j);
   }
 }
 
@@ -109,7 +113,7 @@ inline void mat_vec_dot
  matrix contaning only the upper triangular.
  */
 inline void rank_one_update
-(double * __restrict__ X, double const * __restrict__ x,
+(double * PSQN_RESTRICT X, double const * PSQN_RESTRICT x,
  double const scal, size_t const n)
 noexcept {
   double const * xj = x;
@@ -127,8 +131,8 @@ noexcept {
  where X is a symmetric matrix contaning only the upper triangular.
  */
 inline void bfgs_update
-  (double * __restrict__ X, double const * __restrict__ x,
-   double const * __restrict__ H_y, double const y_H_y,
+  (double * PSQN_RESTRICT X, double const * PSQN_RESTRICT x,
+   double const * PSQN_RESTRICT H_y, double const y_H_y,
    double const scal, size_t const n)
   noexcept {
   double const * xj = x,
