@@ -1,5 +1,5 @@
 // we will use openMP to perform the comptutation in parallel
-// [[Rcpp::plugins(openmp)]]
+// [[Rcpp::plugins(openmp, cpp11)]]
 
 // we use RcppArmadillo to simplify the code
 // [[Rcpp::depends(RcppArmadillo)]]
@@ -72,7 +72,7 @@ public:
    @param gr gradient vector with respect to global and private parameters.
    */
   double grad
-    (double const * __restrict__ point, double * __restrict__ gr) const {
+    (double const * point, double * gr) const {
     arma::vec const beta = vec_no_cp(point           , X.n_rows),
                        u = vec_no_cp(point + X.n_rows, Z.n_rows);
 
@@ -148,6 +148,8 @@ SEXP get_mlogit_optimizer(List data, unsigned const max_threads){
  @param strong_wolfe true if the strong Wolfe condition should be used.
  @param max_cg maximum number of conjugate gradient iterations in each
  iteration. Use zero if there should not be a limit.
+ @param pre_method preconditioning method in conjugate gradient method.
+ zero yields no preconditioning and one yields diagonal preconditioning.
  */
 // [[Rcpp::export]]
 List optim_mlogit
@@ -155,7 +157,7 @@ List optim_mlogit
    unsigned const n_threads, double const c1,
    double const c2, bool const use_bfgs = true, int const trace = 0L,
    double const cg_tol = .5, bool const strong_wolfe = true,
-   size_t const max_cg = 0L){
+   size_t const max_cg = 0L, int const pre_method = 1L){
   XPtr<mlogit_topim> optim(ptr);
 
   // check that we pass a parameter value of the right length
@@ -165,7 +167,8 @@ List optim_mlogit
   NumericVector par = clone(val);
   optim->set_n_threads(n_threads);
   auto res = optim->optim(&par[0], rel_eps, max_it, c1, c2,
-                          use_bfgs, trace, cg_tol, strong_wolfe, max_cg);
+                          use_bfgs, trace, cg_tol, strong_wolfe, max_cg,
+                          static_cast<PSQN::precondition>(pre_method));
   NumericVector counts = NumericVector::create(
     res.n_eval, res.n_grad,  res.n_cg);
   counts.names() = CharacterVector::create("function", "gradient", "n_cg");
